@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
+ * Binocle <http://binocle.com> (author: Olivier Letz <oletz@binocle.com>)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +22,9 @@
 #include "config.h"
 
 #include <vector>
+#include <sstream>
+#include <cstdio>
+#include <cctype>
 #include <unistd.h>
 
 #include "gettext.h"
@@ -96,6 +100,12 @@ void controller::process_events()
     /* default: do nothing */
 }
 
+bool controller::allow_early_quit()
+{
+    /* default: allow to quit when there is nothing to do */
+    return true;
+}
+
 
 dispatch::dispatch(int* argc, char** argv,
         bool equalizer, bool equalizer_3d, bool equalizer_slave_node,
@@ -167,12 +177,15 @@ void dispatch::init(const open_input_data& input_data)
         _video_output = new video_output_qt();
     }
     if ((_eq && !_eq_slave_node) || !_gui_mode) {
-        if (input_data.urls.size() == 0)
-            throw exc(_("No video to play."));
-        std::ostringstream v;
-        s11n::save(v, input_data);
-        controller::send_cmd(command::open, v.str());
-        controller::send_cmd(command::toggle_play);
+        if (input_data.urls.size() == 0) {
+            if (early_quit_is_allowed())
+                throw exc(_("No video to play."));
+        } else {
+            std::ostringstream v;
+            s11n::save(v, input_data);
+            controller::send_cmd(command::open, v.str());
+            controller::send_cmd(command::toggle_play);
+        }
     }
 }
 
@@ -195,22 +208,21 @@ void dispatch::step()
 {
     assert(global_dispatch);
     if (global_dispatch->_playing) {
-        bool more = global_dispatch->_player->run_step();
-        if (!more) {
-            global_dispatch->_playing = false;
-            global_dispatch->_pausing = false;
-            global_dispatch->notify_all(notification::play);            
-            if (!global_dispatch->_gui_mode) {
-                global_dispatch->notify_all(notification::quit);
-            } else {
-                global_dispatch->force_stop();
-            }
-        }
+        if (!global_dispatch->_player->run_step())
+            global_dispatch->stop_player();
     } else {
         usleep(1000);
     }
 }
 
+bool dispatch::early_quit_is_allowed() const
+{
+    for (size_t i = 0; i < _controllers.size(); i++)
+        if (!_controllers[i]->allow_early_quit())
+            return false;
+    return true;
+}
+
 void dispatch::visit_all_controllers(int action, const notification& note) const
 {
     std::vector<controller*> visited_controllers;
@@ -293,11 +305,6 @@ class open_input_data* dispatch::get_input_data()
     return &_input_data;
 }
 
-class player* dispatch::get_player()
-{
-    return _player;
-}
-
 void dispatch::set_playing(bool p)
 {
     _playing = p;
@@ -362,23 +369,38 @@ static float clamp(float x, float lo, float hi)
     return std::min(std::max(x, lo), hi);
 }
 
-void dispatch::force_stop()
+void dispatch::stop_player()
+{
+    bool early_quit = !_gui_mode && early_quit_is_allowed();
+    force_stop(!early_quit);
+    notify_all(notification::play);
+    if (early_quit)
+        notify_all(notification::quit);
+}
+
+void dispatch::force_stop(bool reopen_media_input)
 {
     if (_player) {
         _player->close();
-        delete _player;
+        if (!_eq)
+            delete _player;     // The equalizer code will delete the player object at the appropriate time
         _player = NULL;
     }
     if (_media_input) {
         // close, reopen, and reinitialize media input
         _media_input->close();
-        _media_input->open(_input_data.urls, _input_data.dev_request);
-        _media_input->set_stereo_layout(_parameters.stereo_layout(), _parameters.stereo_layout_swap());
-        _media_input->select_video_stream(_parameters.video_stream());
-        if (_media_input->audio_streams() > 0)
-            _media_input->select_audio_stream(_parameters.audio_stream());
-        if (_media_input->subtitle_streams() > 0 && _parameters.subtitle_stream() >= 0)
-            _media_input->select_subtitle_stream(_parameters.subtitle_stream());
+        if (reopen_media_input) {
+            _media_input->open(_input_data.urls, _input_data.dev_request);
+            _media_input->set_stereo_layout(_parameters.stereo_layout(), _parameters.stereo_layout_swap());
+            _media_input->select_video_stream(_parameters.video_stream());
+            if (_media_input->audio_streams() > 0)
+                _media_input->select_audio_stream(_parameters.audio_stream());
+            if (_media_input->subtitle_streams() > 0 && _parameters.subtitle_stream() >= 0)
+                _media_input->select_subtitle_stream(_parameters.subtitle_stream());
+        } else {
+            delete _media_input;
+            _media_input = NULL;
+        }
     }
     if (_video_output) {
         _video_output->deinit();
@@ -398,9 +420,13 @@ void dispatch::receive_cmd(const command& cmd)
     {
     case command::noop:
         break;
+    case command::quit:
+        force_stop(false);
+        notify_all(notification::quit);
+        break;
     // Play state
     case command::open:
-        force_stop();
+        force_stop(false);
         notify_all(notification::play);
         s11n::load(p, _input_data);
         // Create media input
@@ -488,7 +514,7 @@ void dispatch::receive_cmd(const command& cmd)
         notify_all(notification::open);
         break;
     case command::close:
-        force_stop();
+        force_stop(false);
         notify_all(notification::play);
         notify_all(notification::open);
         break;
@@ -602,6 +628,10 @@ void dispatch::receive_cmd(const command& cmd)
         _parameters.set_fullscreen_inhibit_screensaver(s11n::load<bool>(p));
         notify_all(notification::fullscreen_inhibit_screensaver);
         break;
+    case command::set_fullscreen_3d_ready_sync:
+        _parameters.set_fullscreen_3d_ready_sync(s11n::load<bool>(p));
+        notify_all(notification::fullscreen_3d_ready_sync);
+        break;
     case command::adjust_contrast:
         _parameters.set_contrast(clamp(_parameters.contrast() + s11n::load<float>(p), -1.0f, +1.0f));
         notify_all(notification::contrast);
@@ -663,58 +693,85 @@ void dispatch::receive_cmd(const command& cmd)
         notify_all(notification::subtitle_size);
         break;
     case command::set_subtitle_scale:
-        _parameters.set_subtitle_scale(std::max(s11n::load<float>(p), 0.0f));
+        _parameters.set_subtitle_scale(s11n::load<float>(p));
         notify_all(notification::subtitle_scale);
         break;
     case command::set_subtitle_color:
         _parameters.set_subtitle_color(s11n::load<uint64_t>(p));
         notify_all(notification::subtitle_color);
         break;
+    case command::set_subtitle_shadow:
+        _parameters.set_subtitle_shadow(s11n::load<int>(p));
+        notify_all(notification::subtitle_shadow);
+        break;
+#if HAVE_LIBXNVCTRL
+    case command::set_sdi_output_format:
+        _parameters.set_sdi_output_format(s11n::load<int>(p));
+        notify_all(notification::sdi_output_format);
+        break;
+    case command::set_sdi_output_left_stereo_mode:
+        _parameters.set_sdi_output_left_stereo_mode(static_cast<parameters::stereo_mode_t>(s11n::load<int>(p)));
+        notify_all(notification::sdi_output_left_stereo_mode);
+        break;
+    case command::set_sdi_output_right_stereo_mode:
+        _parameters.set_sdi_output_right_stereo_mode(static_cast<parameters::stereo_mode_t>(s11n::load<int>(p)));
+        notify_all(notification::sdi_output_right_stereo_mode);
+        break;
+#endif // HAVE_LIBXNVCTRL
     // Per-Video parameters
     case command::cycle_video_stream:
-        if (_player && _media_input->video_streams() > 1
+        if (_media_input && _media_input->video_streams() > 1
                 && parameters().stereo_layout() != parameters::layout_separate) {
             int s = parameters().video_stream() + 1;
             if (s >= _media_input->video_streams())
                 s = 0;
-            _parameters.set_video_stream(_player->set_video_stream(s));
+            s = _player->set_video_stream(s);
+            _parameters.set_video_stream(s);
             notify_all(notification::video_stream);
         }
         break;
     case command::set_video_stream:
-        if (_player && _media_input->video_streams() > 1
+        if (_media_input && _media_input->video_streams() > 1
                 && parameters().stereo_layout() != parameters::layout_separate) {
-            _parameters.set_video_stream(_player->set_video_stream(s11n::load<int>(p)));
+            int s = s11n::load<int>(p);
+            s = _player->set_video_stream(s);
+            _parameters.set_video_stream(s);
             notify_all(notification::video_stream);
         }
         break;
     case command::cycle_audio_stream:
-        if (_player && _media_input->audio_streams() > 1) {
+        if (_media_input && _media_input->audio_streams() > 1) {
             int s = parameters().audio_stream() + 1;
             if (s >= _media_input->audio_streams())
                 s = 0;
-            _parameters.set_audio_stream(_player->set_audio_stream(s));
+            s = _player->set_audio_stream(s);
+            _parameters.set_audio_stream(s);
             notify_all(notification::audio_stream);
         }
         break;
     case command::set_audio_stream:
-        if (_player && _media_input->audio_streams() > 1) {
-            _parameters.set_audio_stream(_player->set_audio_stream(s11n::load<int>(p)));
+        if (_media_input && _media_input->audio_streams() > 1) {
+            int s = s11n::load<int>(p);
+            s = _player->set_audio_stream(s);
+            _parameters.set_audio_stream(s);
             notify_all(notification::audio_stream);
         }
         break;
     case command::cycle_subtitle_stream:
-        if (_player && _media_input->subtitle_streams() > 0) {
+        if (_media_input && _media_input->subtitle_streams() > 0) {
             int s = parameters().subtitle_stream() + 1;
             if (s >= _media_input->subtitle_streams())
                 s = -1;
-            _parameters.set_subtitle_stream(_player->set_subtitle_stream(s));
+            s = _player->set_subtitle_stream(s);
+            _parameters.set_subtitle_stream(s);
             notify_all(notification::subtitle_stream);
         }
         break;
     case command::set_subtitle_stream:
-        if (_player && _media_input->subtitle_streams() > 0) {
-            _parameters.set_subtitle_stream(_player->set_subtitle_stream(s11n::load<int>(p)));
+        if (_media_input && _media_input->subtitle_streams() > 0) {
+            int s = s11n::load<int>(p);
+            s = _player->set_subtitle_stream(s);
+            _parameters.set_subtitle_stream(s);
             notify_all(notification::subtitle_stream);
         }
         break;
@@ -800,6 +857,9 @@ void dispatch::receive_cmd(const command& cmd)
         _parameters.set_audio_mute(!_parameters.audio_mute());
         notify_all(notification::audio_mute);
         break;
+    case command::update_display_pos:
+        notify_all(notification::display_pos);
+        break;
     }
 }
 
@@ -823,3 +883,333 @@ void dispatch::load_state(const std::string& s)
     s11n::load(iss, _pausing);
     s11n::load(iss, _position);
 }
+
+static bool parse_bool(const std::string& s, bool* x)
+{
+    if (s == "1" || s == "on" || s == "true") {
+        *x = true;
+        return true;
+    } else if (s == "0" || s == "off" || s == "false") {
+        *x = false;
+        return true;
+    } else {
+        return false;
+    }
+}
+
+static bool parse_aspect_ratio(const std::string& s, float* x)
+{
+    std::vector<std::string> tokens = str::tokens(s, ":");
+    float xn, xd;
+    if (tokens.size() == 2 && str::to(tokens[0], &xn) && str::to(tokens[1], &xd)) {
+        *x = std::min(std::max(xn / xd, 1.0f), 2.39f);
+        return true;
+    } else if (tokens.size() == 1 && str::to(s, x)) {
+        *x = std::min(std::max(*x, 1.0f), 2.39f);
+        return true;
+    } else {
+        return false;
+    }
+}
+
+static bool parse_open_input_data(const std::vector<std::string>& tokens, open_input_data* oid)
+{
+    for (size_t i = 1; i < tokens.size(); i++) {
+        if (tokens[i].length() > 2 && tokens[i].substr(0, 2) == "--") {
+            std::vector<std::string> subtokens = str::tokens(tokens[i].substr(2), "=");
+            if (subtokens.size() == 2 && subtokens[0] == "device-type") {
+                if (subtokens[1] == "none") {
+                    oid->dev_request.device = device_request::no_device;
+                    continue;
+                } else if (subtokens[1] == "default") {
+                    oid->dev_request.device = device_request::sys_default;
+                    continue;
+                } else if (subtokens[1] == "firewire") {
+                    oid->dev_request.device = device_request::firewire;
+                    continue;
+                } else if (subtokens[1] == "x11") {
+                    oid->dev_request.device = device_request::x11;
+                    continue;
+                }
+            } else if (subtokens.size() == 2 && subtokens[0] == "device-frame-size") {
+                std::vector<std::string> wh = str::tokens(subtokens[1], "x");
+                if (wh.size() == 2
+                        && str::to(wh[0], &oid->dev_request.width) && oid->dev_request.width >= 0
+                        && str::to(wh[1], &oid->dev_request.height) && oid->dev_request.height >= 0)
+                    continue;
+            } else if (subtokens.size() == 2 && subtokens[0] == "device-frame-rate") {
+                std::vector<std::string> nd = str::tokens(subtokens[1], "/");
+                if (nd.size() == 2
+                        && str::to(nd[0], &oid->dev_request.frame_rate_num) && oid->dev_request.frame_rate_num >= 0
+                        && str::to(nd[1], &oid->dev_request.frame_rate_den) && oid->dev_request.frame_rate_den >= 0)
+                    continue;
+            } else if (subtokens.size() == 2 && subtokens[0] == "device-format") {
+                if (subtokens[1] == "default") {
+                    oid->dev_request.request_mjpeg = false;
+                    continue;
+                } else if (subtokens[1] == "mjpeg") {
+                    oid->dev_request.request_mjpeg = true;
+                    continue;
+                }
+            }
+            return false;
+        } else {
+            // URLs use percent-encoding; see http://en.wikipedia.org/wiki/Percent_encoding
+            std::string url;
+            for (size_t j = 0; j < tokens[i].size(); j++) {
+                if (tokens[i][j] == '%') {
+                    if (j + 2 < tokens[i].size()) {
+                        char h0 = tokens[i][j + 1];
+                        char h1 = tokens[i][j + 2];
+                        int k = 0;
+                        if (h0 >= '0' && h0 <= '9') {
+                            k = (h0 - '0') << 4;
+                        } else if (h0 >= 'a' && h0 <= 'f') {
+                            k = (h0 - 'a' + 10) << 4;
+                        } else if (h0 >= 'A' && h0 <= 'F') {
+                            k = (h0 - 'A' + 10) << 4;
+                        } else {
+                            return false;
+                        }
+                        if (h1 >= '0' && h1 <= '9') {
+                            k += (h1 - '0');
+                        } else if (h1 >= 'a' && h1 <= 'f') {
+                            k += (h1 - 'a' + 10);
+                        } else if (h1 >= 'A' && h1 <= 'F') {
+                            k += (h1 - 'A' + 10);
+                        } else {
+                            return false;
+                        }
+                        if (k <= 127 && !iscntrl(static_cast<unsigned char>(k))) {
+                            url.push_back(k);
+                            j += 2;
+                        } else {
+                            return false;
+                        }
+                    } else {
+                        return false;
+                    }
+                } else {
+                    url.push_back(tokens[i][j]);
+                }
+            }
+            oid->urls.push_back(url);
+        }
+    }
+    return true;
+}
+
+bool dispatch::parse_command(const std::string& s, command* c)
+{
+    std::vector<std::string> tokens = str::tokens(s, " \t\r");
+    bool ok = true;
+    union {
+        bool b;
+        int i;
+        float f;
+        uint64_t c;
+    } p;
+    float p_crosstalk[3];
+    parameters::stereo_layout_t p_stereo_layout;
+    parameters::stereo_mode_t p_stereo_mode;
+    open_input_data p_oid;
+
+    /* Please keep this in the same order as commands are declared in dispatch.h */
+
+    if (tokens.empty() || tokens[0][0] == '#') {
+        // a comment
+        *c = command(command::noop);
+    } else if (tokens.size() == 1 && tokens[0] == "quit") {
+        *c = command(command::quit);
+    } else if (tokens.size() > 1 && tokens[0] == "open"
+            && parse_open_input_data(tokens, &p_oid)) {
+        std::ostringstream v;
+        s11n::save(v, p_oid);
+        *c = command(command::open, v.str());
+    } else if (tokens.size() == 1 && tokens[0] == "close") {
+        *c = command(command::close);
+    } else if (tokens.size() == 1 && tokens[0] == "toggle-play") {
+        *c = command(command::toggle_play);
+    } else if (tokens.size() == 1 && tokens[0] == "play") {     // extra command, mainly for LIRC
+        if (!dispatch::playing())
+            *c = command(command::toggle_play);
+        else if (dispatch::pausing())
+            *c = command(command::toggle_pause);
+        else
+            *c = command(command::noop);
+    } else if (tokens.size() == 1 && tokens[0] == "stop") {     // extra command, mainly for LIRC
+        if (dispatch::playing())
+            *c = command(command::toggle_play);
+        else
+            *c = command(command::noop);
+    } else if (tokens.size() == 1 && tokens[0] == "toggle-pause") {
+        *c = command(command::toggle_pause);
+    } else if (tokens.size() == 1 && tokens[0] == "pause") {    // extra command, mainly for LIRC
+        if (dispatch::playing() && !dispatch::pausing())
+            *c = command(command::toggle_pause);
+        else
+            *c = command(command::noop);
+    } else if (tokens.size() == 1 && tokens[0] == "step") {
+        *c = command(command::step);
+    } else if (tokens.size() == 2 && tokens[0] == "seek"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::seek, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-pos"
+            && str::to(tokens[1], &p.f) && p.f >= 0.0f && p.f <= 1.0f) {
+        *c = command(command::set_pos, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-audio-device"
+            && str::to(tokens[1], &p.i)) {
+        *c = command(command::set_audio_device, p.i);
+    } else if (tokens.size() == 2 && tokens[0] == "set-stereo-mode"
+            && parameters::parse_stereo_mode(tokens[1], &p_stereo_mode)) {
+        *c = command(command::set_stereo_mode, static_cast<int>(p_stereo_mode));
+    } else if (tokens.size() == 2 && tokens[0] == "set-stereo-mode-swap"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_stereo_mode_swap, p.b);
+    } else if (tokens.size() == 1 && tokens[0] == "toggle-stereo-mode-swap") {
+        *c = command(command::toggle_stereo_mode_swap);
+    } else if (tokens.size() == 4 && tokens[0] == "set-crosstalk"
+            && str::to(tokens[1], p_crosstalk + 0) && p_crosstalk[0] >= 0.0f && p_crosstalk[0] <= 1.0f
+            && str::to(tokens[2], p_crosstalk + 1) && p_crosstalk[1] >= 0.0f && p_crosstalk[1] <= 1.0f
+            && str::to(tokens[3], p_crosstalk + 2) && p_crosstalk[2] >= 0.0f && p_crosstalk[2] <= 1.0f) {
+        std::ostringstream v;
+        s11n::save(v, p_crosstalk[0]);
+        s11n::save(v, p_crosstalk[1]);
+        s11n::save(v, p_crosstalk[2]);
+        *c = command(command::set_crosstalk, v.str());
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-screens"
+            && str::to(tokens[1], &p.i)) {
+        *c = command(command::set_fullscreen_screens, p.i);
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-flip-left"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_fullscreen_flip_left, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-flop-left"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_fullscreen_flop_left, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-flip-right"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_fullscreen_flip_right, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-flop-right"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_fullscreen_flop_right, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-inhibit-screensaver"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_fullscreen_inhibit_screensaver, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-fullscreen-3dr-sync"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_fullscreen_3d_ready_sync, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-contrast"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_contrast, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-contrast"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_contrast, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-brightness"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_brightness, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-brightness"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_brightness, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-hue"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_hue, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-hue"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_hue, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-saturation"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_saturation, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-saturation"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_saturation, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-zoom"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_zoom, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-zoom"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_zoom, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-loop-mode"
+            && (tokens[1] == "off" || tokens[1] == "current")) {
+        parameters::loop_mode_t l = (tokens[1] == "off"
+                ? parameters::no_loop : parameters::loop_current);
+        *c = command(command::set_loop_mode, static_cast<int>(l));
+    } else if (tokens.size() == 2 && tokens[0] == "set-audio-delay"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_audio_delay, p.f);
+    } else if ((tokens.size() == 1 || tokens.size() == 2) && tokens[0] == "set-subtitle-encoding") {
+        std::ostringstream v;
+        s11n::save(v, tokens.size() > 1 ? tokens[1] : std::string(""));
+        *c = command(command::set_subtitle_encoding, v.str());
+    } else if (tokens.size() >= 1 && tokens[0] == "set-subtitle-font") {
+        // A font name can contain spaces and therefore span multiple tokens.
+        std::ostringstream v;
+        s11n::save(v, tokens.size() > 1 ? str::trim(str::trim(s).substr(17)) : std::string(""));
+        *c = command(command::set_subtitle_font, v.str());
+    } else if (tokens.size() == 2 && tokens[0] == "set-subtitle-size"
+            && str::to(tokens[1], &p.i)) {
+        *c = command(command::set_subtitle_size, p.i);
+    } else if (tokens.size() == 2 && tokens[0] == "set-subtitle-color"
+            && str::to(tokens[1], &p.c)) {
+        *c = command(command::set_subtitle_color, p.c);
+    } else if (tokens.size() == 2 && tokens[0] == "set-subtitle-shadow"
+            && str::to(tokens[1], &p.i)) {
+        *c = command(command::set_subtitle_shadow, p.i);
+    } else if (tokens.size() == 2 && tokens[0] == "set-video-stream"
+            && str::to(tokens[1], &p.i) && p.i >= 0) {
+        *c = command(command::set_video_stream, p.i);
+    } else if (tokens.size() == 1 && tokens[0] == "cycle-video-stream") {
+        *c = command(command::cycle_video_stream);
+    } else if (tokens.size() == 2 && tokens[0] == "set-audio-stream"
+            && str::to(tokens[1], &p.i) && p.i >= 0) {
+        *c = command(command::set_audio_stream, p.i);
+    } else if (tokens.size() == 1 && tokens[0] == "cycle-audio-stream") {
+        *c = command(command::cycle_audio_stream);
+    } else if (tokens.size() == 2 && tokens[0] == "set-subtitle-stream"
+            && str::to(tokens[1], &p.i) && p.i >= -1) {
+        *c = command(command::set_subtitle_stream, p.i);
+    } else if (tokens.size() == 1 && tokens[0] == "cycle-subtitle-stream") {
+        *c = command(command::cycle_subtitle_stream);
+    } else if (tokens.size() == 2 && tokens[0] == "set-stereo-layout"
+            && parameters::parse_stereo_layout(tokens[1], &p_stereo_layout)) {
+        *c = command(command::set_stereo_layout, static_cast<int>(p_stereo_layout));
+    } else if (tokens.size() == 2 && tokens[0] == "set-stereo-layout-swap"
+            && parse_bool(tokens[1], &p.b)) {
+        *c = command(command::set_stereo_layout_swap, p.b);
+    } else if (tokens.size() == 2 && tokens[0] == "set-crop-aspect-ratio"
+            && parse_aspect_ratio(tokens[1], &p.f)) {
+        *c = command(command::set_crop_aspect_ratio, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-parallax"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_parallax, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-parallax"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_parallax, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-ghostbust"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_ghostbust, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-ghostbust"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_ghostbust, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "set-subtitle-parallax"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_subtitle_parallax, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-subtitle-parallax"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_subtitle_parallax, p.f);
+    } else if (tokens.size() == 1 && tokens[0] == "toggle-fullscreen") {
+        *c = command(command::toggle_fullscreen);
+    } else if (tokens.size() == 1 && tokens[0] == "center") {
+        *c = command(command::center);
+    } else if (tokens.size() == 2 && tokens[0] == "set-audio-volume"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::set_audio_volume, p.f);
+    } else if (tokens.size() == 2 && tokens[0] == "adjust-audio-volume"
+            && str::to(tokens[1], &p.f)) {
+        *c = command(command::adjust_audio_volume, p.f);
+    } else if (tokens.size() == 1 && tokens[0] == "toggle-audio-mute") {
+        *c = command(command::toggle_audio_mute);
+    } else {
+        ok = false;
+    }
+    return ok;
+}
