@@ -4,6 +4,7 @@
  * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Joe <cuchac@email.cz>
+ * Binocle <http://binocle.com> (author: Olivier Letz <oletz@binocle.com>)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,6 +79,7 @@ public:
     enum type
     {
         noop,                           // no parameters
+        quit,                           // no parameters
         // Play state
         open,                           // open_input_data
         close,                          // no parameters
@@ -98,16 +100,17 @@ public:
         set_fullscreen_flip_right,      // bool
         set_fullscreen_flop_right,      // bool
         set_fullscreen_inhibit_screensaver,     // bool
-        adjust_contrast,                // float (relative adjustment)
+        set_fullscreen_3d_ready_sync,   // bool
         set_contrast,                   // float (absolute value)
-        adjust_brightness,              // float (relative adjustment)
+        adjust_contrast,                // float (relative adjustment)
         set_brightness,                 // float (absolute value)
-        adjust_hue,                     // float (relative adjustment)
+        adjust_brightness,              // float (relative adjustment)
         set_hue,                        // float (absolute value)
-        adjust_saturation,              // float (relative adjustment)
+        adjust_hue,                     // float (relative adjustment)
         set_saturation,                 // float (absolute value)
-        adjust_zoom,                    // float (relative adjustment)
+        adjust_saturation,              // float (relative adjustment)
         set_zoom,                       // float (absolute value)
+        adjust_zoom,                    // float (relative adjustment)
         set_loop_mode,                  // parameters::loop_mode_t
         set_audio_delay,                // float (absolute value)
         set_subtitle_encoding,          // string (encoding name)
@@ -115,28 +118,35 @@ public:
         set_subtitle_size,              // int
         set_subtitle_scale,             // float
         set_subtitle_color,             // uint64_t
+        set_subtitle_shadow,            // int
+#if HAVE_LIBXNVCTRL
+        set_sdi_output_format,          // int
+        set_sdi_output_left_stereo_mode,  // parameters::stereo_mode_t
+        set_sdi_output_right_stereo_mode, // parameters::stereo_mode_t
+#endif // HAVE_LIBXNVCTRL
         // Per-Video parameters
-        cycle_video_stream,             // no parameters
         set_video_stream,               // int
-        cycle_audio_stream,             // no parameters
+        cycle_video_stream,             // no parameters
         set_audio_stream,               // int
-        cycle_subtitle_stream,          // no parameters
+        cycle_audio_stream,             // no parameters
         set_subtitle_stream,            // int
+        cycle_subtitle_stream,          // no parameters
         set_stereo_layout,              // video_frame::stereo_layout
         set_stereo_layout_swap,         // bool
         set_crop_aspect_ratio,          // float
-        adjust_parallax,                // float (relative adjustment)
         set_parallax,                   // float (absolute value)
-        adjust_ghostbust,               // float (relative adjustment)
+        adjust_parallax,                // float (relative adjustment)
         set_ghostbust,                  // float (absolute value)
-        adjust_subtitle_parallax,       // float (relative adjustment)
+        adjust_ghostbust,               // float (relative adjustment)
         set_subtitle_parallax,          // float (absolute value)
+        adjust_subtitle_parallax,       // float (relative adjustment)
         // Volatile parameters
         toggle_fullscreen,              // no parameters
         center,                         // no parameters
-        adjust_audio_volume,            // float (relative adjustment)
         set_audio_volume,               // float (absolute value)
+        adjust_audio_volume,            // float (relative adjustment)
         toggle_audio_mute,              // no parameters
+        update_display_pos,             // no parameters
     };
     
     type type;
@@ -216,6 +226,7 @@ public:
         fullscreen_flip_right,
         fullscreen_flop_right,
         fullscreen_inhibit_screensaver,
+        fullscreen_3d_ready_sync,
         contrast,
         brightness,
         hue,
@@ -228,6 +239,12 @@ public:
         subtitle_size,
         subtitle_scale,
         subtitle_color,
+        subtitle_shadow,
+#if HAVE_LIBXNVCTRL
+        sdi_output_format,
+        sdi_output_left_stereo_mode,
+        sdi_output_right_stereo_mode,
+#endif // HAVE_LIBXNVCTRL
         // Per-Video parameters
         video_stream,
         audio_stream,
@@ -243,6 +260,7 @@ public:
         center,
         audio_volume,
         audio_mute,
+        display_pos,
     };
 
     const enum type type;
@@ -278,6 +296,12 @@ public:
     /* The controller is asked to process events via this function. The default
      * implementation simply does nothing. */
     virtual void process_events();
+
+    /* The controller can prevent Bino from exiting when there is no video
+     * to play. In this case, the following function should return 'false'.
+     * This is intended to be used for controllers that might send another
+     * 'open' command in the future. */
+    virtual bool allow_early_quit();
 };
 
 // The dispatch (singleton).
@@ -309,8 +333,10 @@ private:
     bool _pausing;
     float _position;
 
-    void force_stop();
+    void stop_player();
+    void force_stop(bool reopen_media_input = true);
 
+    bool early_quit_is_allowed() const;
     void visit_all_controllers(int action, const notification& note) const;
     void notify_all(const notification& note);
 
@@ -354,9 +380,13 @@ public:
 
     /* Interface for Equalizer. */
     class open_input_data* get_input_data();
-    class player* get_player();
     std::string save_state() const;
     void load_state(const std::string& s);
+    void stop_eq_player() { stop_player(); }
+
+    /* Helper function for text-based controllers: parse a command from a string.
+     * Return false if this fails, otherwise store the command in c. */
+    static bool parse_command(const std::string& s, command* c);
 };
 
 #endif
