@@ -6,6 +6,7 @@
  * Stefan Eilemann <eile@eyescale.ch>
  * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  * Joe <cuchac@email.cz>
+ * Binocle <http://binocle.com> (author: Olivier Letz <oletz@binocle.com>)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,11 +59,16 @@
 #if HAVE_LIBEQUALIZER
 # include "player_equalizer.h"
 #endif
+#include "command_file.h"
 #if HAVE_LIBLIRCCLIENT
 # include "lirc.h"
 #endif
 #include "lib_versions.h"
 
+#if HAVE_LIBXNVCTRL
+#include "NVCtrl.h"
+#endif // HAVE_LIBXNVCTRL
+
 
 /* Return the directory containing our locale data (translated messages). */
 static const char *localedir()
@@ -233,6 +239,13 @@ int main(int argc, char *argv[])
     opt::tuple<int> device_frame_rate("device-frame-rate", '\0', opt::optional,
             1, std::numeric_limits<int>::max(), std::vector<int>(2, 0), 2, "/");
     options.push_back(&device_frame_rate);
+    std::vector<std::string> device_formats;
+    device_formats.push_back("default");
+    device_formats.push_back("mjpeg");
+    opt::val<std::string> device_format("device-format", '\0', opt::optional, device_formats, "");
+    options.push_back(&device_format);
+    opt::val<std::string> read_commands("read-commands", '\0', opt::optional);
+    options.push_back(&read_commands);
     opt::val<std::string> lirc_config("lirc-config", '\0', opt::optional);
     options.push_back(&lirc_config);
     std::vector<std::string> input_modes;
@@ -259,36 +272,7 @@ int main(int argc, char *argv[])
     options.push_back(&audio);
     opt::val<int> subtitle("subtitle", 's', opt::optional, 0, 999, 0);
     options.push_back(&subtitle);
-    std::vector<std::string> video_output_modes;
-    video_output_modes.push_back("stereo");
-    video_output_modes.push_back("alternating");
-    video_output_modes.push_back("mono-left");
-    video_output_modes.push_back("mono-right");
-    video_output_modes.push_back("top-bottom");
-    video_output_modes.push_back("top-bottom-half");
-    video_output_modes.push_back("left-right");
-    video_output_modes.push_back("left-right-half");
-    video_output_modes.push_back("even-odd-rows");
-    video_output_modes.push_back("even-odd-columns");
-    video_output_modes.push_back("checkerboard");
-    video_output_modes.push_back("hdmi-frame-pack");
-    video_output_modes.push_back("red-cyan-monochrome");
-    video_output_modes.push_back("red-cyan-half-color");
-    video_output_modes.push_back("red-cyan-full-color");
-    video_output_modes.push_back("red-cyan-dubois");
-    video_output_modes.push_back("green-magenta-monochrome");
-    video_output_modes.push_back("green-magenta-half-color");
-    video_output_modes.push_back("green-magenta-full-color");
-    video_output_modes.push_back("green-magenta-dubois");
-    video_output_modes.push_back("amber-blue-monochrome");
-    video_output_modes.push_back("amber-blue-half-color");
-    video_output_modes.push_back("amber-blue-full-color");
-    video_output_modes.push_back("amber-blue-dubois");
-    video_output_modes.push_back("red-green-monochrome");
-    video_output_modes.push_back("red-blue-monochrome");
-    video_output_modes.push_back("equalizer");
-    video_output_modes.push_back("equalizer-3d");
-    opt::val<std::string> video_output_mode("output", 'o', opt::optional, video_output_modes, "");
+    opt::val<std::string> video_output_mode("output", 'o', opt::optional);
     options.push_back(&video_output_mode);
     opt::flag swap_eyes("swap-eyes", 'S', opt::optional);
     options.push_back(&swap_eyes);
@@ -304,6 +288,8 @@ int main(int argc, char *argv[])
     options.push_back(&fullscreen_flip_right);
     opt::flag fullscreen_flop_right("fullscreen-flop-right", '\0', opt::optional);
     options.push_back(&fullscreen_flop_right);
+    opt::flag fullscreen_3d_ready_sync("fullscreen-3d-ready-sync", '\0', opt::optional);
+    options.push_back(&fullscreen_3d_ready_sync);
     opt::val<float> zoom("zoom", 'z', opt::optional, 0.0f, 1.0f);
     options.push_back(&zoom);
     opt::tuple<float> crop_aspect_ratio("crop", 'C', opt::optional, 0.0f, 100.0f, std::vector<float>(), 2, ":");
@@ -320,6 +306,8 @@ int main(int argc, char *argv[])
     options.push_back(&subtitle_scale);
     opt::color subtitle_color("subtitle-color", '\0', opt::optional);
     options.push_back(&subtitle_color);
+    opt::val<int> subtitle_shadow("subtitle-shadow", '\0', opt::optional, -1, 1);
+    options.push_back(&subtitle_shadow);
     opt::val<float> subtitle_parallax("subtitle-parallax", '\0', opt::optional, -1.0f, +1.0f);
     options.push_back(&subtitle_parallax);
     opt::val<float> parallax("parallax", 'P', opt::optional, -1.0f, +1.0f);
@@ -334,6 +322,12 @@ int main(int argc, char *argv[])
     options.push_back(&swap_interval);
     opt::flag loop("loop", 'l', opt::optional);
     options.push_back(&loop);
+#if HAVE_LIBXNVCTRL
+    opt::val<int> sdi_output_format("sdi-output-format", '\0', opt::optional,
+            NV_CTRL_GVIO_VIDEO_FORMAT_487I_59_94_SMPTE259_NTSC,
+            NV_CTRL_GVIO_VIDEO_FORMAT_2048I_47_96_SMPTE372);
+    options.push_back(&sdi_output_format);
+#endif // HAVE_LIBXNVCTRL
     // Accept some Equalizer options. These are passed to Equalizer for interpretation.
     opt::val<std::string> eq_server("eq-server", '\0', opt::optional);
     options.push_back(&eq_server);
@@ -412,6 +406,8 @@ int main(int argc, char *argv[])
                 + "  --device-type=TYPE       " + _("Type of input device: default, firewire, x11.") + '\n'
                 + "  --device-frame-size=WxH  " + _("Request frame size WxH from input device.") + '\n'
                 + "  --device-frame-rate=N/D  " + _("Request frame rate N/D from input device.") + '\n'
+                + "  --device-format=FORMAT   " + _("Request device format 'default' or 'mjpeg'.") + '\n'
+                + "  --read-commands=FILE     " + _("Read commands from a file.") + '\n'
                 + "  --lirc-config=FILE       " + _("Use the given LIRC configuration file.") + '\n'
                 + "                           " + _("This option can be used more than once.") + '\n'
                 + "  -v|--video=STREAM        " + _("Select video stream (1-n, depending on input).") + '\n'
@@ -471,6 +467,7 @@ int main(int argc, char *argv[])
                 + "  --fullscreen-flop-left   " + _("Flop left view horizontally when fullscreen.") + '\n'
                 + "  --fullscreen-flip-right  " + _("Flip right view vertically when fullscreen.") + '\n'
                 + "  --fullscreen-flop-right  " + _("Flop right view horizontally when fullscreen.") + '\n'
+                + "  --fullscreen-3dr-sync    " + _("Use DLP 3-D Ready Sync when fullscreen.") + '\n'
                 + "  -z|--zoom=Z              " + _("Set zoom for wide videos (0=off to 1=full).") + '\n'
                 + "  -C|--crop=W:H            " + _("Crop video to given aspect ratio (0:0=off).") + '\n'
                 + "  -c|--center              " + _("Center window on screen.") + '\n'
@@ -479,6 +476,7 @@ int main(int argc, char *argv[])
                 + "  --subtitle-size=N        " + _("Set subtitle font size.") + '\n'
                 + "  --subtitle-scale=S       " + _("Set subtitle scale factor.") + '\n'
                 + "  --subtitle-color=COLOR   " + _("Set subtitle color, in [AA]RRGGBB format.") + '\n'
+                + "  --subtitle-shadow=-1|0|1 " + _("Set subtitle shadow, -1=default, 0=off, 1=on.") + '\n'
                 + "  --subtitle-parallax=VAL  " + _("Subtitle parallax adjustment (-1 to +1).") + '\n'
                 + "  -P|--parallax=VAL        " + _("Parallax adjustment (-1 to +1).") + '\n'
                 + "  --crosstalk=VAL          " + _("Crosstalk leak level (0 to 1).") + '\n'
@@ -488,6 +486,7 @@ int main(int argc, char *argv[])
                 + "  --swap-interval=D        " + _("Frame rate divisor for display refresh rate.") + '\n'
                 + "                           " + _("Default is 0 for benchmark mode, 1 otherwise.") + '\n'
                 + "  -l|--loop                " + _("Loop the input media.") + '\n'
+                + "  --sdi-output-format=F    " + _("Set SDI output format.") + '\n'
                 + '\n'
                 + _("Interactive control:") + '\n'
                 + "  ESC                      " + _("Leave fullscreen mode, or quit.") + '\n'
@@ -495,7 +494,7 @@ int main(int argc, char *argv[])
                 + "  p / SPACE                " + _("Pause / unpause.") + '\n'
                 + "  f                        " + _("Toggle fullscreen.") + '\n'
                 + "  c                        " + _("Center window.") + '\n'
-                + "  e                        " + _("Swap left/right eye.") + '\n'
+                + "  e / F7                   " + _("Swap left/right eye.") + '\n'
                 + "  v                        " + _("Cycle through available video streams.") + '\n'
                 + "  a                        " + _("Cycle through available audio streams.") + '\n'
                 + "  s                        " + _("Cycle through available subtitle streams.") + '\n'
@@ -598,18 +597,16 @@ int main(int argc, char *argv[])
     if (audio_device.is_set())
         controller::send_cmd(command::set_audio_device, audio_device.value() - 1);
     if (video_output_mode.is_set()) {
+        parameters::stereo_mode_t stereo_mode;
         if (video_output_mode.value() == "equalizer") {
             controller::send_cmd(command::set_stereo_mode, static_cast<int>(parameters::mode_mono_left));
-            controller::send_cmd(command::set_stereo_mode_swap, false);
         } else if (video_output_mode.value() == "equalizer-3d") {
             controller::send_cmd(command::set_stereo_mode, static_cast<int>(parameters::mode_mono_left));
-            controller::send_cmd(command::set_stereo_mode_swap, false);
-        } else {
-            parameters::stereo_mode_t stereo_mode;
-            bool stereo_mode_swap;
-            parameters::stereo_mode_from_string(video_output_mode.value(), stereo_mode, stereo_mode_swap);
+        } else if (parameters::parse_stereo_mode(video_output_mode.value(), &stereo_mode)) {
             controller::send_cmd(command::set_stereo_mode, static_cast<int>(stereo_mode));
-            controller::send_cmd(command::set_stereo_mode_swap, stereo_mode_swap);
+        } else {
+            msg::err(_("Unknown mode %s."), video_output_mode.value().c_str());
+            return 1;
         }
     }
     if (swap_eyes.is_set())
@@ -635,6 +632,8 @@ int main(int argc, char *argv[])
         controller::send_cmd(command::set_fullscreen_flip_right, fullscreen_flip_right.value());
     if (fullscreen_flop_right.is_set())
         controller::send_cmd(command::set_fullscreen_flop_right, fullscreen_flop_right.value());
+    if (fullscreen_3d_ready_sync.is_set())
+        controller::send_cmd(command::set_fullscreen_3d_ready_sync, fullscreen_3d_ready_sync.value());
     if (zoom.is_set())
         controller::send_cmd(command::set_zoom, zoom.value());
     if (loop.is_set())
@@ -657,6 +656,12 @@ int main(int argc, char *argv[])
         controller::send_cmd(command::set_subtitle_scale, subtitle_scale.value());
     if (subtitle_color.is_set())
         controller::send_cmd(command::set_subtitle_color, static_cast<uint64_t>(subtitle_color.value()));
+    if (subtitle_shadow.is_set())
+        controller::send_cmd(command::set_subtitle_shadow, subtitle_shadow.value());
+#if HAVE_LIBXNVCTRL
+    if (sdi_output_format.is_set())
+        controller::send_cmd(command::set_sdi_output_format, sdi_output_format.value());
+#endif // HAVE_LIBXNVCTRL
 
     /* Set volatile parameters */
     if (fullscreen.is_set() && fullscreen.value())
@@ -672,7 +677,7 @@ int main(int argc, char *argv[])
     open_input_data input_data;
     if (device_type.value() == "") {
         input_data.dev_request.device =
-            (arguments.size() == 1 && arguments[0].substr(0, 5) == "/dev/"
+            (arguments.size() >= 1 && arguments[0].substr(0, 5) == "/dev/"
              ? device_request::sys_default : device_request::no_device);
     } else {
         input_data.dev_request.device =
@@ -684,6 +689,8 @@ int main(int argc, char *argv[])
     input_data.dev_request.height = device_frame_size.value()[1];
     input_data.dev_request.frame_rate_num = device_frame_rate.value()[0];
     input_data.dev_request.frame_rate_den = device_frame_rate.value()[1];
+    if (device_format.value() == "mjpeg")
+        input_data.dev_request.request_mjpeg = true;
     input_data.urls = arguments;
     if (video.is_set() > 0)
         input_data.params.set_video_stream(video.value() - 1);
@@ -714,8 +721,12 @@ int main(int argc, char *argv[])
         input_data.params.set_subtitle_parallax(subtitle_parallax.value());
 
     int retval = 0;
+    std::vector<command_file*> command_files;
     try {
-        global_dispatch.init(input_data);
+        for (size_t i = 0; i < read_commands.values().size(); i++) {
+            command_files.push_back(new command_file(read_commands.values()[i]));
+            command_files[i]->init();
+        }
 #if HAVE_LIBLIRCCLIENT
         lircclient lirc(PACKAGE, lirc_config.values());
         try {
@@ -729,6 +740,7 @@ int main(int argc, char *argv[])
             msg::wrn(_("This version of Bino was compiled without support for LIRC."));
         }
 #endif
+        global_dispatch.init(input_data);
         if (dispatch_equalizer) {
 #if HAVE_LIBEQUALIZER
             player_equalizer::mainloop();
@@ -748,6 +760,8 @@ int main(int argc, char *argv[])
         retval = 1;
     }
 
+    for (size_t i = 0; i < command_files.size(); i++)
+        delete command_files[i];
     delete qt_app;
 
     return retval;
