@@ -5,6 +5,7 @@
  * Martin Lambers <marlam@marlam.de>
  * Joe <joe@wpj.cz>
  * D. Matz <bandregent@yahoo.de>
+ * Binocle <http://binocle.com> (author: Olivier Letz <oletz@binocle.com>)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,13 +36,18 @@
 #include "msg.h"
 #include "dbg.h"
 
+#if HAVE_LIBXNVCTRL
+#include "NVCtrl.h"
+#endif // HAVE_LIBXNVCTRL
+
 
 device_request::device_request() :
     device(no_device),
     width(0),
     height(0),
     frame_rate_num(0),
-    frame_rate_den(0)
+    frame_rate_den(0),
+    request_mjpeg(false)
 {
 }
 
@@ -52,6 +58,7 @@ void device_request::save(std::ostream &os) const
     s11n::save(os, height);
     s11n::save(os, frame_rate_num);
     s11n::save(os, frame_rate_den);
+    s11n::save(os, request_mjpeg);
 }
 
 void device_request::load(std::istream &is)
@@ -63,6 +70,7 @@ void device_request::load(std::istream &is)
     s11n::load(is, height);
     s11n::load(is, frame_rate_num);
     s11n::load(is, frame_rate_den);
+    s11n::load(is, request_mjpeg);
 }
 
 
@@ -85,6 +93,7 @@ parameters::parameters()
     unset_fullscreen_flip_right();
     unset_fullscreen_flop_right();
     unset_fullscreen_inhibit_screensaver();
+    unset_fullscreen_3d_ready_sync();
     unset_contrast();
     unset_brightness();
     unset_hue();
@@ -97,6 +106,12 @@ parameters::parameters()
     unset_subtitle_size();
     unset_subtitle_scale();
     unset_subtitle_color();
+    unset_subtitle_shadow();
+#if HAVE_LIBXNVCTRL
+    unset_sdi_output_format();
+    unset_sdi_output_left_stereo_mode();
+    unset_sdi_output_right_stereo_mode();
+#endif // HAVE_LIBXNVCTRL
     // Per-Video parameters
     unset_video_stream();
     unset_audio_stream();
@@ -131,6 +146,7 @@ const bool parameters::_fullscreen_flop_left_default = false;
 const bool parameters::_fullscreen_flip_right_default = false;
 const bool parameters::_fullscreen_flop_right_default = false;
 const bool parameters::_fullscreen_inhibit_screensaver_default = true;
+const bool parameters::_fullscreen_3d_ready_sync_default = false;
 const float parameters::_contrast_default = 0.0f;
 const float parameters::_brightness_default = 0.0f;
 const float parameters::_hue_default = 0.0f;
@@ -143,6 +159,12 @@ const std::string parameters::_subtitle_font_default = "";
 const int parameters::_subtitle_size_default = -1;
 const float parameters::_subtitle_scale_default = -1.0f;
 const uint64_t parameters::_subtitle_color_default = std::numeric_limits<uint64_t>::max();
+const int parameters::_subtitle_shadow_default = -1;
+#if HAVE_LIBXNVCTRL
+const int parameters::_sdi_output_format_default = NV_CTRL_GVIO_VIDEO_FORMAT_1080P_25_00_SMPTE274;
+const parameters::stereo_mode_t parameters::_sdi_output_left_stereo_mode_default = mode_mono_left;
+const parameters::stereo_mode_t parameters::_sdi_output_right_stereo_mode_default = mode_mono_right;
+#endif // HAVE_LIBXNVCTRL
 // Per-Video parameter defaults
 const int parameters::_video_stream_default = 0;
 const int parameters::_audio_stream_default = 0;
@@ -245,6 +267,31 @@ void parameters::stereo_layout_from_string(const std::string &s, stereo_layout_t
     }
 }
 
+bool parameters::parse_stereo_layout(const std::string& s, stereo_layout_t* stereo_layout)
+{
+    bool ok = true;
+    if (s == "mono") {
+        *stereo_layout = layout_mono;
+    } else if (s == "separate-left-right") {
+        *stereo_layout = layout_separate;
+    } else if (s == "alternating-left-right") {
+        *stereo_layout = layout_alternating;
+    } else if (s == "top-bottom") {
+        *stereo_layout = layout_top_bottom;
+    } else if (s == "top-bottom-half") {
+        *stereo_layout = layout_top_bottom_half;
+    } else if (s == "left-right") {
+        *stereo_layout = layout_left_right;
+    } else if (s == "left-right-half") {
+        *stereo_layout = layout_left_right_half;
+    } else if (s == "even-odd-rows") {
+        *stereo_layout = layout_even_odd_rows;
+    } else {
+        ok = false;
+    }
+    return ok;
+}
+
 std::string parameters::stereo_mode_to_string(stereo_mode_t stereo_mode, bool stereo_mode_swap)
 {
     std::string s;
@@ -344,64 +391,74 @@ void parameters::stereo_mode_from_string(const std::string &s, stereo_mode_t &st
     } else {
         t = s;
         stereo_mode_swap = false;
-    } if (t == "stereo") {
-        stereo_mode = mode_stereo;
-    } else if (t == "alternating") {
-        stereo_mode = mode_alternating;
-    } else if (t == "mono-left") {
-        stereo_mode = mode_mono_left;
-    } else if (t == "mono-right") {
-        stereo_mode = mode_mono_right;
-    } else if (t == "top-bottom") {
-        stereo_mode = mode_top_bottom;
-    } else if (t == "top-bottom-half") {
-        stereo_mode = mode_top_bottom_half;
-    } else if (t == "left-right") {
-        stereo_mode = mode_left_right;
-    } else if (t == "left-right-half") {
-        stereo_mode = mode_left_right_half;
-    } else if (t == "even-odd-rows") {
-        stereo_mode = mode_even_odd_rows;
-    } else if (t == "even-odd-columns") {
-        stereo_mode = mode_even_odd_columns;
-    } else if (t == "checkerboard") {
-        stereo_mode = mode_checkerboard;
-    } else if (t == "hdmi-frame-pack") {
-        stereo_mode = mode_hdmi_frame_pack;
-    } else if (t == "red-cyan-monochrome") {
-        stereo_mode = mode_red_cyan_monochrome;
-    } else if (t == "red-cyan-half-color") {
-        stereo_mode = mode_red_cyan_half_color;
-    } else if (t == "red-cyan-full-color") {
-        stereo_mode = mode_red_cyan_full_color;
-    } else if (t == "red-cyan-dubois") {
-        stereo_mode = mode_red_cyan_dubois;
-    } else if (t == "green-magenta-monochrome") {
-        stereo_mode = mode_green_magenta_monochrome;
-    } else if (t == "green-magenta-half-color") {
-        stereo_mode = mode_green_magenta_half_color;
-    } else if (t == "green-magenta-full-color") {
-        stereo_mode = mode_green_magenta_full_color;
-    } else if (t == "green-magenta-dubois") {
-        stereo_mode = mode_green_magenta_dubois;
-    } else if (t == "amber-blue-monochrome") {
-        stereo_mode = mode_amber_blue_monochrome;
-    } else if (t == "amber-blue-half-color") {
-        stereo_mode = mode_amber_blue_half_color;
-    } else if (t == "amber-blue-full-color") {
-        stereo_mode = mode_amber_blue_full_color;
-    } else if (t == "amber-blue-dubois") {
-        stereo_mode = mode_amber_blue_dubois;
-    } else if (t == "red-green-monochrome") {
-        stereo_mode = mode_red_green_monochrome;
-    } else if (t == "red-blue-monochrome") {
-        stereo_mode = mode_red_blue_monochrome;
-    } else {
+    }
+    if (!parse_stereo_mode(t, &stereo_mode)) {
         // safe fallback
         stereo_mode = mode_mono_left;
     }
 }
 
+bool parameters::parse_stereo_mode(const std::string& s, stereo_mode_t* stereo_mode)
+{
+    bool ok = true;
+    if (s == "stereo") {
+        *stereo_mode = mode_stereo;
+    } else if (s == "alternating") {
+        *stereo_mode = mode_alternating;
+    } else if (s == "mono-left") {
+        *stereo_mode = mode_mono_left;
+    } else if (s == "mono-right") {
+        *stereo_mode = mode_mono_right;
+    } else if (s == "top-bottom") {
+        *stereo_mode = mode_top_bottom;
+    } else if (s == "top-bottom-half") {
+        *stereo_mode = mode_top_bottom_half;
+    } else if (s == "left-right") {
+        *stereo_mode = mode_left_right;
+    } else if (s == "left-right-half") {
+        *stereo_mode = mode_left_right_half;
+    } else if (s == "even-odd-rows") {
+        *stereo_mode = mode_even_odd_rows;
+    } else if (s == "even-odd-columns") {
+        *stereo_mode = mode_even_odd_columns;
+    } else if (s == "checkerboard") {
+        *stereo_mode = mode_checkerboard;
+    } else if (s == "hdmi-frame-pack") {
+        *stereo_mode = mode_hdmi_frame_pack;
+    } else if (s == "red-cyan-monochrome") {
+        *stereo_mode = mode_red_cyan_monochrome;
+    } else if (s == "red-cyan-half-color") {
+        *stereo_mode = mode_red_cyan_half_color;
+    } else if (s == "red-cyan-full-color") {
+        *stereo_mode = mode_red_cyan_full_color;
+    } else if (s == "red-cyan-dubois") {
+        *stereo_mode = mode_red_cyan_dubois;
+    } else if (s == "green-magenta-monochrome") {
+        *stereo_mode = mode_green_magenta_monochrome;
+    } else if (s == "green-magenta-half-color") {
+        *stereo_mode = mode_green_magenta_half_color;
+    } else if (s == "green-magenta-full-color") {
+        *stereo_mode = mode_green_magenta_full_color;
+    } else if (s == "green-magenta-dubois") {
+        *stereo_mode = mode_green_magenta_dubois;
+    } else if (s == "amber-blue-monochrome") {
+        *stereo_mode = mode_amber_blue_monochrome;
+    } else if (s == "amber-blue-half-color") {
+        *stereo_mode = mode_amber_blue_half_color;
+    } else if (s == "amber-blue-full-color") {
+        *stereo_mode = mode_amber_blue_full_color;
+    } else if (s == "amber-blue-dubois") {
+        *stereo_mode = mode_amber_blue_dubois;
+    } else if (s == "red-green-monochrome") {
+        *stereo_mode = mode_red_green_monochrome;
+    } else if (s == "red-blue-monochrome") {
+        *stereo_mode = mode_red_blue_monochrome;
+    } else {
+        ok = false;
+    }
+    return ok;
+}
+
 std::string parameters::loop_mode_to_string(loop_mode_t loop_mode)
 {
     if (loop_mode == loop_current) {
@@ -454,6 +511,8 @@ void parameters::save(std::ostream &os) const
     s11n::save(os, _fullscreen_flop_right_set);
     s11n::save(os, _fullscreen_inhibit_screensaver);
     s11n::save(os, _fullscreen_inhibit_screensaver_set);
+    s11n::save(os, _fullscreen_3d_ready_sync);
+    s11n::save(os, _fullscreen_3d_ready_sync_set);
     s11n::save(os, _contrast);
     s11n::save(os, _contrast_set);
     s11n::save(os, _brightness);
@@ -478,6 +537,16 @@ void parameters::save(std::ostream &os) const
     s11n::save(os, _subtitle_scale_set);
     s11n::save(os, _subtitle_color);
     s11n::save(os, _subtitle_color_set);
+    s11n::save(os, _subtitle_shadow);
+    s11n::save(os, _subtitle_shadow_set);
+#if HAVE_LIBXNVCTRL
+    s11n::save(os, _sdi_output_format);
+    s11n::save(os, _sdi_output_format_set);
+    s11n::save(os, static_cast<int>(_sdi_output_left_stereo_mode));
+    s11n::save(os, _sdi_output_left_stereo_mode_set);
+    s11n::save(os, static_cast<int>(_sdi_output_right_stereo_mode));
+    s11n::save(os, _sdi_output_right_stereo_mode_set);
+#endif // HAVE_LIBXNVCTRL
     // Per-Video parameters
     s11n::save(os, _video_stream);
     s11n::save(os, _video_stream_set);
@@ -543,6 +612,8 @@ void parameters::load(std::istream &is)
     s11n::load(is, _fullscreen_flop_right_set);
     s11n::load(is, _fullscreen_inhibit_screensaver);
     s11n::load(is, _fullscreen_inhibit_screensaver_set);
+    s11n::load(is, _fullscreen_3d_ready_sync);
+    s11n::load(is, _fullscreen_3d_ready_sync_set);
     s11n::load(is, _contrast);
     s11n::load(is, _contrast_set);
     s11n::load(is, _brightness);
@@ -567,6 +638,16 @@ void parameters::load(std::istream &is)
     s11n::load(is, _subtitle_scale_set);
     s11n::load(is, _subtitle_color);
     s11n::load(is, _subtitle_color_set);
+    s11n::load(is, _subtitle_shadow);
+    s11n::load(is, _subtitle_shadow_set);
+#if HAVE_LIBXNVCTRL
+    s11n::load(is, _sdi_output_format);
+    s11n::load(is, _sdi_output_format_set);
+    s11n::load(is, x); _sdi_output_left_stereo_mode = static_cast<stereo_mode_t>(x);
+    s11n::load(is, _sdi_output_left_stereo_mode_set);
+    s11n::load(is, x); _sdi_output_right_stereo_mode = static_cast<stereo_mode_t>(x);
+    s11n::load(is, _sdi_output_right_stereo_mode_set);
+#endif // HAVE_LIBXNVCTRL
     // Per-Video parameters
     s11n::load(is, _video_stream);
     s11n::load(is, _video_stream_set);
@@ -622,6 +703,8 @@ std::string parameters::save_session_parameters() const
         s11n::save(oss, "fullscreen_flop_right", fullscreen_flop_right());
     if (!fullscreen_inhibit_screensaver_is_default())
         s11n::save(oss, "fullscreen_inhibit_screensaver", fullscreen_inhibit_screensaver());
+    if (!fullscreen_3d_ready_sync_is_default())
+        s11n::save(oss, "fullscreen_3d_ready_sync", fullscreen_3d_ready_sync());
     if (!contrast_is_default())
         s11n::save(oss, "contrast", contrast());
     if (!brightness_is_default())
@@ -646,6 +729,16 @@ std::string parameters::save_session_parameters() const
         s11n::save(oss, "subtitle_scale", _subtitle_scale);
     if (!subtitle_color_is_default())
         s11n::save(oss, "subtitle_color", _subtitle_color);
+    if (!subtitle_shadow_is_default())
+        s11n::save(oss, "subtitle_shadow", _subtitle_shadow);
+#if HAVE_LIBXNVCTRL
+    if (!sdi_output_format_is_default())
+        s11n::save(oss, "sdi_output_format", sdi_output_format());
+    if (!sdi_output_left_stereo_mode_is_default())
+        s11n::save(oss, "sdi_output_left_stereo_mode", stereo_mode_to_string(sdi_output_left_stereo_mode(), false));
+    if (!sdi_output_right_stereo_mode_is_default())
+        s11n::save(oss, "sdi_output_right_stereo_mode", stereo_mode_to_string(sdi_output_right_stereo_mode(), false));
+#endif // HAVE_LIBXNVCTRL
     return oss.str();
 }
 
@@ -691,6 +784,9 @@ void parameters::load_session_parameters(const std::string &s)
         } else if (name == "fullscreen_inhibit_screensaver") {
             s11n::load(value, _fullscreen_inhibit_screensaver);
             _fullscreen_inhibit_screensaver_set = true;
+        } else if (name == "fullscreen_3d_ready_sync") {
+            s11n::load(value, _fullscreen_3d_ready_sync);
+            _fullscreen_3d_ready_sync_set = true;
         } else if (name == "contrast") {
             s11n::load(value, _contrast);
             _contrast_set = true;
@@ -729,6 +825,26 @@ void parameters::load_session_parameters(const std::string &s)
         } else if (name == "subtitle_color") {
             s11n::load(value, _subtitle_color);
             _subtitle_color_set = true;
+        } else if (name == "subtitle_shadow") {
+            s11n::load(value, _subtitle_shadow);
+            _subtitle_shadow_set = true;
+#if HAVE_LIBXNVCTRL
+        } else if (name == "sdi_output_format") {
+            s11n::load(value, _sdi_output_format);
+            _sdi_output_format_set = true;
+        } else if (name == "sdi_output_left_stereo_mode") {
+            std::string s;
+            bool swap_value;
+            s11n::load(value, s);
+            stereo_mode_from_string(s, _sdi_output_left_stereo_mode, swap_value);
+            _sdi_output_left_stereo_mode_set = true;
+        } else if (name == "sdi_output_right_stereo_mode") {
+            std::string s;
+            bool swap_value;
+            s11n::load(value, s);
+            stereo_mode_from_string(s, _sdi_output_right_stereo_mode, swap_value);
+            _sdi_output_right_stereo_mode_set = true;
+#endif // HAVE_LIBXNVCTRL
         }
     }
 }
