@@ -5,6 +5,7 @@
  * Martin Lambers <marlam@marlam.de>
  * Joe <joe@wpj.cz>
  * D. Matz <bandregent@yahoo.de>
+ * Binocle <http://binocle.com> (author: Olivier Letz <oletz@binocle.com>)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +47,7 @@ public:
     int height;         // Request frames of the given height (0 means default).
     int frame_rate_num; // Request a specific frame rate (0/0 means default).
     int frame_rate_den; // For example 1/25, 1/30, ...
+    bool request_mjpeg; // Request MJPEG format from device
 
     // Constructor
     device_request();
@@ -80,6 +82,7 @@ public:
     // Convert the stereo layout to and from a string representation
     static std::string stereo_layout_to_string(stereo_layout_t stereo_layout, bool stereo_layout_swap);
     static void stereo_layout_from_string(const std::string &s, stereo_layout_t &stereo_layout, bool &stereo_layout_swap);
+    static bool parse_stereo_layout(const std::string& s, stereo_layout_t* stereo_layout);
 
     // Stereo mode: the output mode for left and right view
     typedef enum {
@@ -114,6 +117,7 @@ public:
     // Convert the stereo mode to and from a string representation
     static std::string stereo_mode_to_string(stereo_mode_t stereo_mode, bool stereo_mode_swap);
     static void stereo_mode_from_string(const std::string &s, stereo_mode_t &stereo_mode, bool &stereo_mode_swap);
+    static bool parse_stereo_mode(const std::string& s, stereo_mode_t* stereo_mode);
 
     typedef enum {
         no_loop,                        // Do not loop.
@@ -169,6 +173,7 @@ public:
     PARAMETER(bool, fullscreen_flip_right)    // Flip right view vertically in fullscreen mode
     PARAMETER(bool, fullscreen_flop_right)    // Flop right view horizontally in fullscreen mode
     PARAMETER(bool, fullscreen_inhibit_screensaver)     // Inhibit screensaver when in fullscreen mode
+    PARAMETER(bool, fullscreen_3d_ready_sync) // Use DLP 3-D Ready Sync in fullscreen mode
     PARAMETER(float, contrast)                // Contrast adjustment, -1 .. +1
     PARAMETER(float, brightness)              // Brightness adjustment, -1 .. +1
     PARAMETER(float, hue)                     // Hue adjustment, -1 .. +1
@@ -181,6 +186,12 @@ public:
     PARAMETER(int, subtitle_size)             // Subtitle point size, -1 means keep default
     PARAMETER(float, subtitle_scale)          // Scale factor
     PARAMETER(uint64_t, subtitle_color)       // Subtitle color in uint32_t bgra32 format, > UINT32_MAX means keep default
+    PARAMETER(int, subtitle_shadow)           // Subtitle shadow, -1 = default, 0 = force off, 1 = force on
+#if HAVE_LIBXNVCTRL
+    PARAMETER(int, sdi_output_format)         // SDI output format
+    PARAMETER(stereo_mode_t, sdi_output_left_stereo_mode)  // SDI output left stereo mode
+    PARAMETER(stereo_mode_t, sdi_output_right_stereo_mode) // SDI output right stereo mode
+#endif // HAVE_LIBXNVCTRL
     // Per-Video parameters
     PARAMETER(int, video_stream)              // Video stream index
     PARAMETER(int, audio_stream)              // Audio stream index, or -1 if there is no audio stream
@@ -375,12 +386,17 @@ public:
     // Constructor, Destructor
     subtitle_box();
 
-    // Comparison. This assumes that two subtitle boxes are identical when they
-    // have the same presentation times.
+    // Comparison.
     bool operator==(const subtitle_box &box) const
     {
-        return (presentation_start_time == box.presentation_start_time
-                && presentation_stop_time == box.presentation_stop_time);
+        if (!is_valid() && !box.is_valid()) {
+            return true;
+        } else if (format == image) {
+            return (presentation_start_time == box.presentation_start_time
+                    && presentation_stop_time == box.presentation_stop_time);
+        } else {
+            return style == box.style && str == box.str;
+        }
     }
     bool operator!=(const subtitle_box &box) const
     {
