@@ -270,27 +270,29 @@ bool subtitle_renderer::render_to_display_size(const subtitle_box &box) const
     return (box.format != subtitle_box::image);
 }
 
-void subtitle_renderer::prerender(const subtitle_box &box, int64_t timestamp,
+bool subtitle_renderer::prerender(const subtitle_box &box, int64_t timestamp,
         const parameters &params,
         int width, int height, float pixel_aspect_ratio,
         int &bb_x, int &bb_y, int &bb_w, int &bb_h)
 {
     assert(_initialized);
+    bool r = false;
     _fmt = box.format;
     switch (_fmt)
     {
     case subtitle_box::text:
     case subtitle_box::ass:
-        prerender_ass(box, timestamp, params, width, height, pixel_aspect_ratio);
+        r = prerender_ass(box, timestamp, params, width, height, pixel_aspect_ratio);
         break;
     case subtitle_box::image:
-        prerender_img(box);
+        r = prerender_img(box);
         break;
     }
     bb_x = _bb_x;
     bb_y = _bb_y;
     bb_w = _bb_w;
     bb_h = _bb_h;
+    return r;
 }
 
 void subtitle_renderer::render(uint32_t *bgra32_buffer)
@@ -345,7 +347,6 @@ void subtitle_renderer::blend_ass_image(const ASS_Image *img, uint32_t *buf)
 void subtitle_renderer::set_ass_parameters(const parameters &params)
 {
     std::vector<std::string> overrides;
-    overrides.clear();
     if (!params.subtitle_font_is_default())
     {
         overrides.push_back(std::string("Default.Fontname=") + params.subtitle_font());
@@ -365,10 +366,15 @@ void subtitle_renderer::set_ass_parameters(const parameters &params)
         overrides.push_back(std::string("Default.PrimaryColour=") + color_str);
         overrides.push_back(std::string("Default.SecondaryColour=") + color_str);
     }
+    if (!params.subtitle_shadow_is_default())
+    {
+        overrides.push_back(std::string("Default.Shadow=")
+                + (params.subtitle_shadow() == 0 ? "0" : "3"));
+    }
     const char *ass_overrides[overrides.size() + 1];
     for (size_t i = 0; i < overrides.size(); i++)
     {
-        ass_overrides[i] = ::strdup(overrides[i].c_str());
+        ass_overrides[i] = overrides[i].c_str();
     }
     ass_overrides[overrides.size()] = NULL;
     ass_set_style_overrides(_ass_library, const_cast<char **>(ass_overrides));
@@ -376,7 +382,7 @@ void subtitle_renderer::set_ass_parameters(const parameters &params)
     ass_process_force_style(_ass_track);
 }
 
-void subtitle_renderer::prerender_ass(const subtitle_box &box, int64_t timestamp,
+bool subtitle_renderer::prerender_ass(const subtitle_box &box, int64_t timestamp,
         const parameters &params, int width, int height, float pixel_aspect_ratio)
 {
     // Lock
@@ -441,7 +447,8 @@ void subtitle_renderer::prerender_ass(const subtitle_box &box, int64_t timestamp
     set_ass_parameters(params);
 
     // Render subtitle
-    _ass_img = ass_render_frame(_ass_renderer, _ass_track, timestamp / 1000, NULL);
+    int change_detected = 0;
+    _ass_img = ass_render_frame(_ass_renderer, _ass_track, timestamp / 1000, &change_detected);
 
     // Unlock
     global_libass_mutex.unlock();
@@ -478,6 +485,7 @@ void subtitle_renderer::prerender_ass(const subtitle_box &box, int64_t timestamp
         _bb_y = min_y;
         _bb_h = max_y - min_y + 1;
     }
+    return change_detected;
 }
 
 void subtitle_renderer::render_ass(uint32_t *bgra32_buffer)
@@ -494,7 +502,7 @@ void subtitle_renderer::render_ass(uint32_t *bgra32_buffer)
     }
 }
 
-void subtitle_renderer::prerender_img(const subtitle_box &box)
+bool subtitle_renderer::prerender_img(const subtitle_box &box)
 {
     _img_box = &box;
     // Determine bounding box
@@ -528,6 +536,7 @@ void subtitle_renderer::prerender_img(const subtitle_box &box)
         _bb_y = min_y;
         _bb_h = max_y - min_y + 1;
     }
+    return true;
 }
 
 void subtitle_renderer::render_img(uint32_t *bgra32_buffer)
