@@ -5,6 +5,7 @@
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <frederic.devernay@inrialpes.fr>
  * Joe <cuchac@email.cz>
+ * Binocle <http://binocle.com> (author: Olivier Letz <oletz@binocle.com>)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,10 +66,11 @@
 gl_thread::gl_thread(video_output_qt* vo_qt, video_output_qt_widget* vo_qt_widget) :
     _vo_qt(vo_qt), _vo_qt_widget(vo_qt_widget),
     _render(false),
-    _resize(false),
     _action_activate(false),
     _action_prepare(false),
-    _failure(false)
+    _action_finished(false),
+    _failure(false),
+    _display_frameno(0)
 {
 }
 
@@ -81,35 +83,45 @@ GLXEWContext* gl_thread::glxewGetContext() const
 
 void gl_thread::set_render(bool r)
 {
+    _redisplay = r;
     _render = r;
-    _display_frameno = 0;
-    _pti = 0;
-    _ptc = 0;
 }
 
 void gl_thread::resize(int w, int h)
 {
     _w = w;
     _h = h;
-    _resize = true;
 }
 
 void gl_thread::activate_next_frame()
 {
-    _action_mutex.lock();
+    if (_failure)
+        return;
+    _wait_mutex.lock();
+    _action_finished = false;
     _action_activate = true;
-    _action_cond.wait(_action_mutex);
-    _action_mutex.unlock();
+    _wait_cond.wait(_wait_mutex);
+    _action_finished = true;
+    _wait_mutex.unlock();
 }
 
 void gl_thread::prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle)
 {
-    _action_mutex.lock();
+    if (_failure)
+        return;
+    _wait_mutex.lock();
     _next_subtitle = subtitle;
     _next_frame = frame;
+    _action_finished = false;
     _action_prepare = true;
-    _action_cond.wait(_action_mutex);
-    _action_mutex.unlock();
+    _wait_cond.wait(_wait_mutex);
+    _action_finished = true;
+    _wait_mutex.unlock();
+}
+
+void gl_thread::redisplay()
+{
+    _redisplay = true;
 }
 
 void gl_thread::run()
@@ -119,22 +131,6 @@ void gl_thread::run()
         _vo_qt_widget->makeCurrent();
         assert(QGLContext::currentContext() == _vo_qt_widget->context());
         while (_render) {
-            _action_mutex.lock();
-            if (_action_activate) {
-                _vo_qt->video_output::activate_next_frame();
-                _action_activate = false;
-                _action_cond.wake_one();
-            }
-            if (_action_prepare) {
-                _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
-                _action_prepare = false;
-                _action_cond.wake_one();
-            }
-            _action_mutex.unlock();
-            if (_resize) {
-                _vo_qt->reshape(_w, _h);
-                _resize = false;
-            }
 #ifdef Q_WS_X11
             GLuint counter;
             if (GLXEW_SGI_video_sync && glXGetVideoSyncSGI(&counter) == 0)
@@ -144,17 +140,40 @@ void gl_thread::run()
 #else
             _display_frameno++;
 #endif
-            _vo_qt->display_current_frame(_display_frameno);
-            _vo_qt_widget->swapBuffers();
-            // When the buffer swap returns, the current frame is just now presented on screen.
-            _pt_mutex.lock();
-            _pt[_pti] = timer::get_microseconds(timer::monotonic);
-            _pti++;
-            if (_pti >= _pts)
-                _pti = 0;
-            if (_ptc < _pts)
-                _ptc++;
-            _pt_mutex.unlock();
+            // In alternating mode, we should always present both left and right view of
+            // a stereo video frame before advancing to the next video frame. This means we
+            // can only switch video frames every other output frame.
+            if (dispatch::parameters().stereo_mode() != parameters::mode_alternating
+                    || _display_frameno % 2 == 0) {
+                if (_action_activate) {
+                    _vo_qt->video_output::activate_next_frame();
+                    _action_activate = false;
+                    _wait_cond.wake_one();
+                    _redisplay = true;
+                }
+            }
+            if (_action_prepare) {
+                _vo_qt->video_output::prepare_next_frame(_next_frame, _next_subtitle);
+                _action_prepare = false;
+                _wait_cond.wake_one();
+            }
+            if (_w > 0 && _h > 0
+                    && (_vo_qt->full_display_width() != _w
+                        || _vo_qt->full_display_height() != _h)) {
+                _vo_qt->reshape(_w, _h);
+                _redisplay = true;
+            }
+            if (_redisplay || dispatch::parameters().stereo_mode() == parameters::mode_alternating) {
+                _redisplay = false;
+                _vo_qt->display_current_frame(_display_frameno);
+#if HAVE_LIBXNVCTRL
+                _vo_qt->sdi_output(_display_frameno);
+#endif // HAVE_LIBXNVCTRL
+                _vo_qt_widget->swapBuffers();
+            } else {
+                // do not busy loop
+                usleep(1000);
+            }
         }
     }
     catch (std::exception& e) {
@@ -162,31 +181,19 @@ void gl_thread::run()
         _render = false;
         _failure = true;
     }
+    if (_action_activate || _action_prepare) {
+        while (!_action_finished)
+            _wait_cond.wake_one();
+    }
     _vo_qt_widget->doneCurrent();
 }
 
 int64_t gl_thread::time_to_next_frame_presentation()
 {
-    if (_ptc < _pts)    // no reliable data yet; assume immediate display
-        return 0;
-    _pt_mutex.lock();
-    int last_pti = _pti == 0 ? _pts - 1 : _pti - 1;
-    int64_t last_pt = _pt[last_pti];
-    int64_t presentation_duration = 0;
-    for (int i = 0; i < _pts - 1; i++) {
-        int cpti = _pti - i - 1;
-        if (cpti < 0)
-            cpti += _pts;
-        int ppti = cpti - 1;
-        if (ppti < 0)
-            ppti = _pts - 1;
-        assert(_pt[cpti] - _pt[ppti] >= 0);
-        presentation_duration += _pt[cpti] - _pt[ppti];
-    }
-    presentation_duration /= _pts - 1;
-    _pt_mutex.unlock();
-    int64_t now = timer::get_microseconds(timer::monotonic);
-    return last_pt + presentation_duration - now;
+    // TODO: return a good estimate of the number of microseconds that will
+    // pass until the next buffer swap completes. For now, just assume that
+    // the next frame will display immediately.
+    return 0;
 }
 
 /* The GL widget */
@@ -250,6 +257,16 @@ void video_output_qt_widget::stop_rendering()
     _timer.stop();
 }
 
+void video_output_qt_widget::redisplay()
+{
+    _gl_thread.redisplay();
+}
+
+void video_output_qt_widget::paintEvent(QPaintEvent*)
+{
+    _gl_thread.redisplay();
+}
+
 void video_output_qt_widget::resizeEvent(QResizeEvent* event)
 {
     _width = event->size().width();
@@ -281,6 +298,7 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
         _vo->send_cmd(command::toggle_play);
         break;
     case Qt::Key_E:
+    case Qt::Key_F7:
         _vo->send_cmd(command::toggle_stereo_mode_swap);
         break;
     case Qt::Key_F:
@@ -456,7 +474,7 @@ void video_container_widget::playloop_step()
     }
     catch (std::exception& e) {
         send_cmd(command::close);
-        QMessageBox::critical(this, "Error", e.what());
+        QMessageBox::critical(this, _("Error"), e.what());
     }
 }
 
@@ -485,6 +503,11 @@ QSize video_container_widget::sizeHint() const
     return QSize(_w, _h);
 }
 
+void video_container_widget::moveEvent(QMoveEvent*)
+{
+    send_cmd(command::update_display_pos);
+}
+
 void video_container_widget::closeEvent(QCloseEvent *)
 {
     send_cmd(command::toggle_play);
@@ -625,11 +648,16 @@ int64_t video_output_qt::wait_for_subtitle_renderer()
 
 void video_output_qt::deinit()
 {
-    if (_widget)
-    {
-        _widget->stop_rendering();
-        _widget->makeCurrent();
-        video_output::deinit();
+    exit_fullscreen();
+    if (_widget) {
+        try {
+            _widget->stop_rendering();
+            _widget->makeCurrent();
+            video_output::deinit();
+        }
+        catch (std::exception& e) {
+            QMessageBox::critical(_widget, _("Error"), e.what());
+        }
         delete _widget;
         _widget = NULL;
     }
@@ -746,11 +774,11 @@ void video_output_qt::mouse_toggle_fullscreen()
 void video_output_qt::suspend_screensaver()
 {
 #if defined(Q_WS_X11)
+    _widget->stop_rendering();
     if (QProcess::execute(QString("xdg-screensaver suspend ")
                 + str::from(_container_widget->winId()).c_str()) != 0)
-    {
         msg::wrn(_("Cannot suspend screensaver."));
-    }
+    _widget->start_rendering();
 #elif defined(Q_WS_WIN)
     /* TODO */
 #elif defined(Q_WS_MAC)
@@ -761,11 +789,11 @@ void video_output_qt::suspend_screensaver()
 void video_output_qt::resume_screensaver()
 {
 #if defined(Q_WS_X11)
+    _widget->stop_rendering();
     if (QProcess::execute(QString("xdg-screensaver resume ")
                 + str::from(_container_widget->winId()).c_str()) != 0)
-    {
         msg::wrn(_("Cannot resume screensaver."));
-    }
+    _widget->start_rendering();
 #elif defined(Q_WS_WIN)
     /* TODO */
 #elif defined(Q_WS_MAC)
@@ -964,6 +992,39 @@ void video_output_qt::process_events()
 
 void video_output_qt::receive_notification(const notification& note)
 {
+    /* Redisplay if a parameter was changed that affects the video display. */
+    if (dispatch::playing()
+            && (note.type == notification::stereo_mode
+                || note.type == notification::stereo_mode_swap
+                || note.type == notification::crosstalk
+                || note.type == notification::fullscreen_flip_left
+                || note.type == notification::fullscreen_flop_left
+                || note.type == notification::fullscreen_flip_right
+                || note.type == notification::fullscreen_flop_right
+                || note.type == notification::fullscreen_3d_ready_sync
+                || note.type == notification::contrast
+                || note.type == notification::brightness
+                || note.type == notification::hue
+                || note.type == notification::saturation
+                || note.type == notification::zoom
+#if HAVE_LIBXNVCTRL
+                || note.type == notification::sdi_output_format
+                || note.type == notification::sdi_output_left_stereo_mode
+                || note.type == notification::sdi_output_right_stereo_mode
+#endif // HAVE_LIBXNVCTRL
+                || note.type == notification::crop_aspect_ratio
+                || note.type == notification::parallax
+                || note.type == notification::ghostbust)) {
+        _widget->redisplay();
+    }
+    /* Redisplay if the widget moved and a masking stereo mode is active. */
+    if (dispatch::playing()
+            && note.type == notification::display_pos
+            && (dispatch::parameters().stereo_mode() == parameters::mode_even_odd_rows
+                || dispatch::parameters().stereo_mode() == parameters::mode_even_odd_columns
+                || dispatch::parameters().stereo_mode() == parameters::mode_checkerboard)) {
+        _widget->redisplay();
+    }
     if (note.type == notification::play && !dispatch::playing()) {
         exit_fullscreen();
     }
