@@ -50,27 +50,20 @@ private:
     video_output_qt* _vo_qt;
     video_output_qt_widget* _vo_qt_widget;
     bool _render;
-    bool _resize;
     int _w, _h;
-    mutex _action_mutex;
-    condition _action_cond;
+    mutex _wait_mutex;
+    condition _wait_cond;
     bool _action_activate;
     bool _action_prepare;
+    bool _action_finished;
+    bool _redisplay;
     video_frame _next_frame;
     subtitle_box _next_subtitle;
     bool _failure;
     exc _e;
-
     // The display frame number
     int64_t _display_frameno;
 
-    // For estimation of next display frame presentation time
-    mutex _pt_mutex;
-    static const int _pts = 5;
-    int64_t _pt[_pts];
-    int _ptc;
-    int _pti;
-
 public:
     gl_thread(video_output_qt* vo_qt, video_output_qt_widget* vo_qt_widget);
 
@@ -82,6 +75,7 @@ public:
     void resize(int w, int h);
     void activate_next_frame();
     void prepare_next_frame(const video_frame &frame, const subtitle_box &subtitle);
+    void redisplay();
 
     int64_t time_to_next_frame_presentation();
 
@@ -120,6 +114,7 @@ public:
 
     void start_rendering();
     void stop_rendering();
+    void redisplay();
     class gl_thread* gl_thread()
     {
         return &_gl_thread;
@@ -134,7 +129,7 @@ protected:
     virtual bool event(QEvent* e) { return QWidget::event(e); }
 # endif
 #endif
-    virtual void paintEvent(QPaintEvent*) {}
+    virtual void paintEvent(QPaintEvent* event);
     virtual void resizeEvent(QResizeEvent* event);
     virtual void keyPressEvent(QKeyEvent *event);
     virtual void mouseReleaseEvent(QMouseEvent *event);
@@ -159,6 +154,7 @@ public:
 signals:
 protected:
     virtual QSize sizeHint() const;
+    virtual void moveEvent(QMoveEvent* event);
     virtual void closeEvent(QCloseEvent *event);
     virtual void receive_notification(const notification& note);
 };
